package cz.zcu.kiv.db.dao.jdbc;

import cz.zcu.kiv.db.dao.OsobaDao;
import cz.zcu.kiv.db.dao.jdbc.model.JdbcUdalost;
import cz.zcu.kiv.db.dao.UdalostDao;
import cz.zcu.kiv.db.model.osoby.Osoba;
import cz.zcu.kiv.db.model.udalosti.Udalost;
import java.util.ArrayList;
import java.util.List;
import javax.sql.DataSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.simple.ParameterizedBeanPropertyRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;
import org.springframework.stereotype.Repository;

/**
 * Implementace rozhrani UdalostDao za pouziti Spring Jdbc.
 * Anotace @Repository("udalostDao") ulozi instanci teto tridy do IOC kontejneru springu,
 * kde bude k dispozici pod identifikatorem "udalostDao".
 * <code>UdalostDao uDao = (UdalostDao) ctx.getBean("udalostDao");</code>
 *  
 * @author Radek Muzika
 */

@Repository("udalostDao")
public class UdalostDaoImpl implements UdalostDao {

    /**
     * Anotace @Autowired zajistuje automaticke instanciovani objektu z IOC kontejneru.
     * Coz znamena ze atributu _osobaDao bude v instanci tridy, ulozenem v IOC kontejneru
     * pod identifikatorem "udalostDao" automaticky prirazena instance tridy OsobaDao, ktera
     * je jiz ulozena v IOC kontejneru.
     */
    @Autowired
    private OsobaDao _osobaDao;
    
    /**
     * SimpleJdbcTemplate umoznuje snadnou manipulaci s databazi. Vice viz.
     * dokumentace springu http://static.springframework.org/spring/docs/2.5.x/reference/jdbc.html#jdbc-SimpleJdbcTemplate
     */
    private SimpleJdbcTemplate simpleJdbcTemplate;
    /**
     * SimpleJdbcInsert umoznuje snadne vkladani dat do databaze a nasledne ziskani
     * automaticky generovaneho primarniho klice (pokud takovy je). Vice viz.
     * dokumentace springu http://static.springframework.org/spring/docs/2.5.x/reference/jdbc.html#jdbc-simple-jdbc-insert-1
     */
    private SimpleJdbcInsert udalostInsert;

    /**
     * Anotace @Autowired pouzita u metody init je dalsim moznym pouzitim IOC. Beana z IOC kontejneru
     * je vlozena do volani metody jako parametr. V tomto pripade se jedna o dataSource,
     * nadefinovany v konfiguracnim souboru.
     * @param dataSource
     */
    @Autowired
    public void init(DataSource dataSource) {
        this.simpleJdbcTemplate = new SimpleJdbcTemplate(dataSource);
        this.udalostInsert = new SimpleJdbcInsert(dataSource)
                .withTableName("udalosti")
                .usingGeneratedKeyColumns("id");
    }

    public List<Udalost> getUdalosti() {
        return this.getUdalosti(false);
    }
    
    public List<Udalost> getUdalosti(boolean vcetneNeaktivnich) {
        String sql = "select * from udalosti "+
                (vcetneNeaktivnich?"":" where aktivni=1")+
                " order by datum_zacatek desc";
        /* Dotaz je mapovan na tridu JdbcUdalost jelikoz potrebujeme ziskaz i foreign keys
         * ktere nejsou jako atributy v domenove tride. 
         * ParameterizedBeanPropertyRowMapper namapuje vysledky dotazu na atributy tridy.*/
        List<JdbcUdalost> jdbclst = this.simpleJdbcTemplate.query(
                sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcUdalost.class));
        List<Udalost> lst = new ArrayList<Udalost>();
        for(JdbcUdalost u: jdbclst) {
            /* JdbcUdalosti musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. Udalost potrebuje jen tridu OsobaDao a
             * to k nacteni odpovedne osoby a kontaktni osoby.*/
            u.setOsobaDao(_osobaDao);
            lst.add(u);
        }
        return lst;
    }

    public Udalost getUdalost(int id) {
        String sql = "select * from udalosti where id = ?";
        try {
            /* Dotaz je mapovan na tridu JdbcUdalost jelikoz potrebujeme ziskaz i foreign keys
             * ktere nejsou jako atributy v domenove tride. */
            JdbcUdalost udalost = this.simpleJdbcTemplate.queryForObject(
                    sql,
                    ParameterizedBeanPropertyRowMapper.newInstance(JdbcUdalost.class),
                    id);
            /* JdbcUdalosti musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. Udalost potrebuje jen tridu OsobaDao a
             * to k nacteni odpovedne osoby a kontaktni osoby.*/
            udalost.setOsobaDao(_osobaDao);
            return udalost;
        } catch (org.springframework.dao.EmptyResultDataAccessException ex) {
            /* Pokud neni pozadovany zaznam v databazi, je vracena vyjimka EmptyResultDataAccessException.
             * V takovem pripade vraci metoda null. */
            return null;
        }
    }

    public List<Udalost> getUdalostiDleOdpovedneOsoby(Osoba osoba) {
        return this.getUdalostiDleOdpovedneOsoby(osoba, false);
    }
    
    public List<Udalost> getUdalostiDleOdpovedneOsoby(Osoba osoba, boolean vcetneNeaktivnich) {
        String sql = "select * from udalosti where fk_osoba=?"+
                (vcetneNeaktivnich?"":" and aktivni=1")+
                " order by datum_zacatek desc ";
        /* Dotaz je mapovan na tridu JdbcUdalost jelikoz potrebujeme ziskaz i foreign keys
         * ktere nejsou jako atributy v domenove tride. */
        List<JdbcUdalost> jdbclst = this.simpleJdbcTemplate.query(
                sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcUdalost.class),
                osoba.getId());
        List<Udalost> lst = new ArrayList<Udalost>();
        for(JdbcUdalost u: jdbclst) {
            /* JdbcUdalosti musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. Udalost potrebuje jen tridu OsobaDao a
             * to k nacteni odpovedne osoby a kontaktni osoby.*/
            u.setOsobaDao(_osobaDao);
            lst.add(u);
        }
        return lst;
    }
    
    public List<Udalost> getUdalostiDleKontaktniOsoby(Osoba osoba) {
        return this.getUdalostiDleKontaktniOsoby(osoba, false);
    }
    
    public List<Udalost> getUdalostiDleKontaktniOsoby(Osoba osoba, boolean vcetneNeaktivnich) {
        String sql = "select * from udalosti where fk_kontaktni_osoba=?"+
                (vcetneNeaktivnich?"":" and aktivni=1")+
                " order by datum_zacatek desc ";
        /* Dotaz je mapovan na tridu JdbcUdalost jelikoz potrebujeme ziskaz i foreign keys
         * ktere nejsou jako atributy v domenove tride. */
        List<JdbcUdalost> jdbclst = this.simpleJdbcTemplate.query(sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcUdalost.class),
                osoba.getId());
        List<Udalost> lst = new ArrayList<Udalost>();
        for(JdbcUdalost u: jdbclst) {
            /* JdbcUdalosti musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. Udalost potrebuje jen tridu OsobaDao a
             * to k nacteni odpovedne osoby a kontaktni osoby.*/
            u.setOsobaDao(_osobaDao);
            lst.add(u);
        }
        return lst;
    }

    /**
     * Pokud je udalost novym zaznamem, dojde ke vlozeni zaznamu do databaze a to pomoci objektu
     * udalostInsert. Pokud zaznam jiz existuje (ma nenulove id) dojde k aktualizaci zaznamu. 
     * @param udalost
     */
    public void ulozUdalost(Udalost udalost) {
        if (udalost.isNovyZaznam()) {
            udalost.setId(udalostInsert.executeAndReturnKey(
                    this.getUdalostParameterSource(udalost)).intValue());
        } else {
            String sql = "update udalosti set " +
                    "nazev_cz=:nazev_cz, " +
                    "nazev_en=:nazev_en, " +
                    "popis_cz=:popis_cz, " +
                    "popis_en=:popis_en, " +
                    "misto_cz=:misto_cz, " +
                    "misto_en=:misto_en, " +
                    "url=:url, " +
                    "stala=:stala, " +
                    "fk_osoba=:fk_osoba," +
                    "fk_kontaktni_osoba=:fk_kontaktni_osoba, " +
                    "datum_zacatek=:datum_zacatek " +
                    "where id=:id";
            this.simpleJdbcTemplate.update(
                    sql,
                    this.getUdalostParameterSource(udalost));
        }
    }

    /**
     * MapSqlParameterSource slouzi k namapovani objektu na jmenne parametry sql dotazu
     * (Napriklad: select * from udalosti where nazev_cz=:nazev_cz)
     * 
     * @param udalost
     * @return namapovany objekt
     */
    private MapSqlParameterSource getUdalostParameterSource(Udalost udalost) {
        return new MapSqlParameterSource()
                .addValue("id", udalost.getId())
                .addValue("nazev_cz", udalost.getNazevCz())
                .addValue("popis_cz", udalost.getPopisCz())
                .addValue("misto_cz", udalost.getMistoCz())
                .addValue("nazev_en", udalost.getNazevEn())
                .addValue("popis_en", udalost.getPopisEn())
                .addValue("misto_en", udalost.getMistoEn())
                .addValue("url", udalost.getUrl())
                .addValue("stala", udalost.isStala()?1:0)
                .addValue("fk_osoba", udalost.getOdpovednaOsoba() != null ? 
                    udalost.getOdpovednaOsoba().getId() : new Integer(0))
                .addValue("fk_kontaktni_osoba", udalost.getKontaktniOsoba() != null ? 
                    udalost.getKontaktniOsoba().getId() : new Integer(0))
                .addValue("autor_zaznamu", udalost.getZaznamEditor())
                .addValue("datum_zacatek", udalost.getDatumZacatek());
    }

    /**
     * Smaze zaznam z databaze.
     * @param udalost
     */
    public void smazUdalost(Udalost udalost) {
        String sql = "delete from udalosti where id=?";
        this.simpleJdbcTemplate.update(sql, udalost.getId());
    }

    /**
     * Nastavi sloupec "aktivni" u daneho zaznamu na hodnotu 0
     * @param udalost
     */
    public void deaktivujUdalost(Udalost udalost) {
        String sql = "update udalosti set aktivni=0 where id=:id";
        this.simpleJdbcTemplate.update(sql, udalost.getId());
    }
    
}
